/*
 * SpaceControlCanvas.java
 *
 * Created on October 30, 2005, 8:49 AM
 */

package Current.canvases.ParameterSpace;

import Current.gui.*;
import Current.manage.*;

import java.awt.*;
//import java.awt.geom.*;
import java.awt.event.*;

import org.pat.graphics.boxed.ListenString;


/**
 *
 * @author  pat
 */
public class SpaceControlCanvas extends DBCanvas
implements MouseListener {
    ListenString[] buttons;
    ListenString help;
    SpaceCanvas SC;
    Manager M;
    
    int padding=4;
    
    // Colors
    Color buttonBg, buttonFg, selectedBg;
    
    // The currently selected mode
    int mode;
    
    /** Creates a new instance of SpaceControlCanvas */
    public SpaceControlCanvas(SpaceCanvas SC, Manager M) {
        this.SC=SC;
        this.M=M;
        M.addListener(this);
        
        //setFont(new Font("sanserif", Font.PLAIN, 12));
        setFont(new Font("Helvetica",Font.PLAIN,12));
        addMouseListener(this);
        
        buttons=new ListenString[7];
        
        double x=padding,y=padding;
        buttons[Mode.STANDARD]=new ListenString("standard", this);
        buttons[Mode.STANDARD].setLeft(x);
        buttons[Mode.STANDARD].setTop(y);
        
        x=buttons[Mode.STANDARD].getRight()+padding;
        buttons[Mode.STANDARD2]=new ListenString("[  ]", this);
        buttons[Mode.STANDARD2].setLeft(x);
        buttons[Mode.STANDARD2].setTop(y);
        
        x=buttons[1].getRight()+padding;
        buttons[Mode.COLOR]=new ListenString("color", this);
        buttons[Mode.COLOR].setLeft(x);
        buttons[Mode.COLOR].setTop(y);
        
        x=buttons[Mode.COLOR].getRight()+padding;
        buttons[Mode.FIT]=new ListenString("fit", this);
        buttons[Mode.FIT].setLeft(x);
        buttons[Mode.FIT].setTop(y);
        
        x=padding;
        y=buttons[Mode.STANDARD].getBottom()+padding;
        buttons[Mode.RAISE]=new ListenString("raise", this);
        buttons[Mode.RAISE].setLeft(x);
        buttons[Mode.RAISE].setTop(y);
        
        x=buttons[Mode.RAISE].getRight()+padding;
        buttons[Mode.LOWER]=new ListenString("lower", this);
        buttons[Mode.LOWER].setLeft(x);
        buttons[Mode.LOWER].setTop(y);
        
        x=buttons[Mode.LOWER].getRight()+padding;
        buttons[Mode.ERASE]=new ListenString("erase", this);
        buttons[Mode.ERASE].setLeft(x);
        buttons[Mode.ERASE].setTop(y);
        
        x=buttons[Mode.ERASE].getRight()+padding;
        //added by Rich
        help=new ListenString("?", this);
        help.setLeft(x);
        help.setTop(y);
        
        
        
        //Setup Colors
        buttonBg=new Color(100,250,100);
        buttonFg=Color.black;
        selectedBg=new Color(255,255,0);
        setBackground(new Color(0,100,0));
        
        for (int i=0; i<buttons.length; i++){
            buttons[i].bgColor=buttonBg;
            buttons[i].fgColor=buttonFg;
        }
        help.bgColor=buttonBg;
        help.fgColor=buttonFg;
        
        mode=SC.mode;
        buttons[mode].bgColor=selectedBg;
    }
    
    /** set the canvas size to its minimal appropriate size. */
    public void setSize(){
        setSize(
        (int)(buttons[6].getRight()+padding),
        (int)(buttons[3].getBottom()+padding));
    }
    
    public void paint(Graphics gfx) {
        Graphics2D g=(Graphics2D) gfx;
        
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
        RenderingHints.VALUE_ANTIALIAS_ON);
        //g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
        //RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
        Dimension D=getSize();
        g.setColor(new Color(0,255,255));
        g.drawRect(0,0,D.width-1,D.height-1);
        for (int i=0; i<buttons.length; i++){
            buttons[i].render(g);
        }
        help.render(g);
    }
    
    public void setMode(int i) {
        if ((mode>=0)&&(mode<buttons.length)) {
            buttons[mode].bgColor=buttonBg;
        }
        if ((i>=0)&&(i<buttons.length)) {
            buttons[i].bgColor=selectedBg;
        }
        mode=i;
        SC.mode=i;
        repaint();        
    }
    
    public void mouseClicked(MouseEvent e) {
        if (help.contains(e.getX(),e.getY())) {
            document();  //added by Rich
            return;
        }
        for (int i=0; i<buttons.length; i++){
            if (buttons[i].contains(e.getX(),e.getY())) {
                setMode(i);
                return;
            }
        }
    }
    
    public void mouseEntered(MouseEvent e) {
    }
    
    public void mouseExited(MouseEvent e) {
    }
    
    public void mousePressed(MouseEvent e) {
    }
    
    public void mouseReleased(MouseEvent e) {
    }
    
    
    public void document() {
        M.mcbSend(new HelpDocument("SpaceControlCanvas.html"));
        //M.setExplain("This window lets you control the effect of clicking button 2 in the main plot window.  Assuming that you have already drawn some orbit tiles, you have the following options for interacting with them:\n\nstandard: when you click on a tile this tile is highlighted and the applet stores the word associated with the tile. \n\ncolor: let's you recolor the tile according to the color on the color selector.\n\n raise: raises the tile in relation to other plotted tiles.\n\nlower: lowers the tile in connetion to other plotter tiles.\n\nerase: erases the tile.\n\nfit: this button resizes the tile to fit well on the screen.\n\n[]:  right now this is essentially the same as standard. We're going to add another function here soon.");
    }
    
    public void mcbReceive(Mode m) {
        //System.out.println("recieved mode");
        setMode(m.getMode());
    }
    
    
}
