/*
 * DisplayBilliardPath.java
 *
 * Created on March 10, 2006, 10:09 AM
 */

package Current.popups.BilliardPath;

import Current.popups.Unfold.*;
import Current.basic.*;
import org.pat.graphics.boxed.ListenString;
import Current.gui.*;
import Current.popups.ImageSave.ImageSaver;
import Current.manage.*;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;


/**
 *
 * @author  pat
 */
public class DisplayBilliardPath extends DBCanvas
implements ImageSaver, UnfoldingData.UnfoldingDataWatcher {
    ListenTriangle T;
    
    double pos=0.5; // position of path in strip.
    
    protected Manager M;
    public DisplayProperties properties;
    protected UnfoldingData UD;
    protected boolean popup;
    //DisplayContainer DC;
    
    ListenString[] labels=new ListenString[3];
    
    
    /** called as an independent popup */
    public DisplayBilliardPath(Manager M) {
        popup=true;
        UnfoldingData ud=new UnfoldingData(M);
        ud.setWatcher(this);
        init(M,ud,new DisplayProperties(M));
    }
    
    /** to be called from a DisplayContainer */
    public DisplayBilliardPath(Manager M, UnfoldingData UD, DisplayProperties properties) {
        popup=false;
        init(M,UD,properties);
    }
    
    private void init(Manager M, UnfoldingData UD, DisplayProperties properties) {
        setFont(new Font("sanserif", Font.PLAIN, 20));
        
        for (int i=0; i<3; i++) {
            labels[i]=new ListenString(""+(1+i), this);
            labels[i].showBorder=false;
        }
        this.M=M;
        this.UD=UD;
        this.properties=properties;
        
        setSize(200,200);
        
        M.ISM.add("billiard path", this);
        M.addListener(this);
    }
    
    /** called when the color settings are changed */
    public void mcbReceive(DisplayProperties.ColorSettings cs) {
        repaint();
    }
    
    public void mcbCleanup() {
        M.ISM.remove(this);
        M.remove(this);
        if (popup) {
            M.remove(UD);
        }
    }
    
    public void paint(Graphics gfx){
        AffineTransform rot=new AffineTransform();
        rot.rotate(properties.rotation_angle);
        
        if (properties.colors.bgColor!=this.getBackground()) {
            setBackground(properties.colors.bgColor);
            gfx.setColor(properties.colors.bgColor);
            gfx.fillRect(0,0, getWidth(), getHeight());
        }
        pos=properties.pos;
        
        for (int i=0; i<3; i++) {
            labels[i].setColors(properties.colors.textColor,properties.colors.bgColor);
        }
        
        
        Graphics2D g=(Graphics2D) gfx;
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
        RenderingHints.VALUE_ANTIALIAS_ON);
        
        T=ListenTriangle.init(M.getZ());
        GeneralPath triangle=T.getPath();
        triangle.transform(rot);
        
        Rectangle2D rect=triangle.getBounds2D();
        double
        s1=getWidth()/rect.getWidth(),
        s2=getHeight()/rect.getHeight(),s;
        if (s1<s2) {
            s=s1;
        } else {
            s=s2;
        }
        triangle=T.getPath();
        
        AffineTransform aff=new AffineTransform();
        aff.translate(getWidth()/2.0, getHeight()/2.0);
        aff.scale(0.95,0.95);
        aff.scale(s,s);
        aff.translate(-rect.getCenterX(),-rect.getCenterY());
        aff.concatenate(rot);
        
        if (properties.edge_labels) { // draw labels
            Complex z;
            Point2D.Double p;
            
            z=(T.z[1].plus(T.z[2])).times(0.5);
            p=new Point2D.Double(z.x,z.y);
            aff.transform(p,p);
            labels[0].setLeft(p.x);
            labels[0].setTop(p.y);
            labels[0].render(g);
            
            z=(T.z[2].plus(T.z[0])).times(0.5);
            p=new Point2D.Double(z.x,z.y);
            aff.transform(p,p);
            labels[1].setLeft(p.x);
            labels[1].setBottom(p.y);
            labels[1].render(g);
            
            z=(T.z[0].plus(T.z[1])).times(0.5);
            p=new Point2D.Double(z.x,z.y);
            aff.transform(p,p);
            labels[2].setRight(p.x);
            labels[2].setVerticalCenter(p.y);
            labels[2].render(g);
        }
        
        
        g.setColor(properties.colors.insideColor1);
        g.fill(aff.createTransformedShape(triangle));
        g.setColor(properties.colors.borderColor);
        g.setStroke(properties.triangleStroke);
        g.draw(aff.createTransformedShape(triangle));
        
        
        
        if (UD.good){
            double y=pos*(UD.top-UD.bot)+UD.bot,
            ratio;
            Complex loc;
            
            GeneralPath path=new GeneralPath();
            
            for (int i=0; i<UD.word.length(); i++) {
                ratio=(y-UD.T[i].z[UD.Bot[i]].y)/(UD.T[i].z[UD.Top[i]].y-UD.T[i].z[UD.Bot[i]].y);
                loc=(T.z[UD.Top[i]].minus(T.z[UD.Bot[i]])).times(ratio).plus(T.z[UD.Bot[i]]);
                if (i==0) {
                    path.moveTo((float)loc.x,(float)loc.y);
                } else {
                    path.lineTo((float)loc.x,(float)loc.y);
                }
            }
            path.closePath();
            g.setColor(properties.colors.pathColor);
            g.setStroke(properties.pathStroke);
            g.draw(aff.createTransformedShape(path));
        }
    }
    
    public void imageSave(Graphics G) {
        paint(G);
    }
    
    public void updatedUnfoldingData() {
        repaint();
    }
}