package Current.popups.Deg100;
import Current.*;
import Current.basic.*;
import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;
import java.math.*;

//The largest denominator we see is 2^{19} and so this is what we multiply by*/

public class Deg100IntegerPolygon {
    BigInteger[] X=new BigInteger[8];
    BigInteger[] Y=new BigInteger[8];
    int n;



    public Deg100IntegerPolygon() {}


    public Deg100IntegerPolygon(Deg100Polygon P) {
	this.n=P.n;
	for(int i=0;i<P.n;++i) {
	    X[i]=amplify(P.k1[i],P.n1[i]);
	    Y[i]=amplify(P.k2[i],P.n2[i]);
	}
    }

    public static BigInteger amplify(int k,int n) {
	Integer K=new Integer(k);
	BigInteger BIG=new BigInteger("2");
	BIG=BIG.pow(19-n);
	BigInteger X=new BigInteger(K.toString());
        X=X.multiply(BIG);
	return(X);
    }


    public Deg100IntegerPolygon(DyadicSquare Q) {

	this.n=4;
	BigInteger X=amplify(Q.x[0],Q.x[1]);
	int yy=(int)(Math.pow(2,Q.y[1]))-Q.y[0];
	BigInteger Y=amplify(yy,Q.y[1]);
	BigInteger R=new BigInteger("2");
	R=R.pow(19-1-Q.k);

	this.X[0]=X.subtract(R);
	this.Y[0]=Y.subtract(R);

	this.X[1]=X.add(R);
	this.Y[1]=Y.subtract(R);

	this.X[2]=X.add(R);
	this.Y[2]=Y.add(R);

	this.X[3]=X.subtract(R);
	this.Y[3]=Y.add(R);
    }




    public static int orientation(BigInteger x1,BigInteger y1,BigInteger x2,BigInteger y2,BigInteger x3,BigInteger y3)
{
    BigInteger X1=BigInteger.ZERO;
    X1=X1.add(x1.multiply(y2));
    X1=X1.add(x2.multiply(y3));
    X1=X1.add(x3.multiply(y1));

    BigInteger X2=BigInteger.ZERO;
    X2=X2.add(y1.multiply(x2));
    X2=X2.add(y2.multiply(x3));
    X2=X2.add(y3.multiply(x1));
    return(X1.compareTo(X2));
}

    //tests if these points come in order on a line segment
    public static int between(BigInteger x1,BigInteger y1,BigInteger x2,BigInteger y2,BigInteger x3,BigInteger y3) {
	int test=orientation(x1,y1,x2,y2,x3,y3);
	if(test!=0) return(0);  //not collinear
	if(x1.compareTo(x2)*x2.compareTo(x3)==1) return(1);  //X ordering is correct, all unequal
	if(y1.compareTo(y2)*y2.compareTo(y3)==1) return(1);  //Y ordering is correct, all unequal
	if((x1.compareTo(x2)==0)&&(y1.compareTo(y2)==0)) return(1); //first two points agree
	if((x3.compareTo(x2)==0)&&(y3.compareTo(y2)==0)) return(1); //last two points agree
	return(0);
    }





    //checks if the point (x,y) is on the polygon
    public static int checkOn(BigInteger x,BigInteger y,Deg100IntegerPolygon P) {
    int local=0;
  for(int i=0;i<P.n;++i) {
      int j=i+1;
      if(j==P.n) j=0; 
      local=between(P.X[i],P.Y[i],x,y,P.X[j],P.Y[j]);
      if(local==1) return(1);

  }
  return(0);
}


    //checks if the point (x,y) is inside the polygon
    public static int checkInside(BigInteger x,BigInteger y,Deg100IntegerPolygon P) {
  int total,local;
  total=0;
  local=0;
  for(int i=0;i<P.n;++i) {
      int j=i+1;
      if(j==P.n) j=0;
      local=orientation(x,y,P.X[i],P.Y[i],P.X[j],P.Y[j]);
      total=total+local;
  }
  if (total<0) total=-total;
  if(total<P.n) return(0);
  return(1);
}


    //checks if the point (x,y) is outside the polygon

   public static int checkOutside(BigInteger x,BigInteger y,Deg100IntegerPolygon P) {
       if(checkInside(x,y,P)==1) return(0);
       if(checkOn(x,y,P)==1) return(0);
       return(1);
   }



    /**this routine tests if polygon1 is contained
       inside polygon 2. 
       Return of 1 means yes.*/

   public static int checkContains(Deg100IntegerPolygon P1,Deg100IntegerPolygon P2) {
	for(int i=0;i<P1.n;++i) {
	  
	    if(checkOutside(P1.X[i],P1.Y[i],P2)==1) return(0);
	}
	return(1);
   }

    /**this checks if at least one vertex of the first polygon is contained in
       the second polygon */

   public static int checkContainsSomeVertex(Deg100IntegerPolygon P1,Deg100IntegerPolygon P2) {
	for(int i=0;i<P1.n;++i) {
	    if(checkOutside(P1.X[i],P1.Y[i],P2)==0) return(1);
	}
	return(0);
   }



/**this routine checks if the 
  line segment with endpoints z1,z2
  intersects the dyadic line segment
  with endpoints z3,z4. 
  0 means segments are disjoint
  1 means that a near-degeneracy caused the test to fail.
  2 means there is a genuine crossing 
  Here z1 stands for (X1,Y1), etc*/


    public static int checkCross(BigInteger X1,BigInteger Y1,BigInteger X2,BigInteger Y2,BigInteger X3,BigInteger Y3,BigInteger X4,BigInteger Y4) {
  int test1,test2;
  test1=orientation(X1,Y1,X3,Y3,X4,Y4)*orientation(X2,Y2,X3,Y3,X4,Y4);
  test2=orientation(X1,Y1,X2,Y2,X4,Y4)*orientation(X1,Y1,X2,Y2,X3,Y3);

  if(between(X1,Y1,X3,Y3,X2,Y2)==1) return(1);  
  if(between(X1,Y1,X4,Y4,X2,Y2)==1) return(1);
  if(between(X3,Y3,X1,Y1,X4,Y4)==1) return(1);  
  if(between(X3,Y3,X2,Y2,X4,Y4)==1) return(1);

  if((test1==-1)&&(test2==-1)) return(2);
  return(0);
}



/**this routine returns
0 if the polygons intersect
1 if the first polygon is contained in the second one
2 if the second polygon is contained in the first one
3 if the polygons are disjoint
*/

    public static int checkDisjoint(Deg100IntegerPolygon P1,Deg100IntegerPolygon P2) {


	if(checkContains(P2,P1)==1) return(2);
	if(checkContains(P1,P2)==1) return(1);


	if(checkContainsSomeVertex(P1,P2)==1) return(0);
     //finally, test for disjointness


	int n1=P1.n;
	int n2=P2.n;
	int test=0;

	BigInteger X1=new BigInteger("0");	
	BigInteger X2=new BigInteger("0");	
	BigInteger X3=new BigInteger("0");	
	BigInteger X4=new BigInteger("0");	
	BigInteger Y1=new BigInteger("0");	
	BigInteger Y2=new BigInteger("0");	
	BigInteger Y3=new BigInteger("0");	
	BigInteger Y4=new BigInteger("0");				      

  for(int j=0;j<n2;++j) {
  for(int i=0;i<n1;++i) {
      int ii=i+1;
      if(ii==n1) ii=0;
      int jj=j+1;
      if(jj==n2) jj=0;
      X1=P1.X[i];  
      Y1=P1.Y[i];
      X2=P1.X[ii];  
      Y2=P1.Y[ii];

      X3=P2.X[j];  
      Y3=P2.Y[j];
      X4=P2.X[jj];  
      Y4=P2.Y[jj];

      test=checkCross(X1,Y1,X2,Y2,X3,Y3,X4,Y4);
      if(test!=0) return(0);
  }}

     return(3);
    }

    public static int checkDisjoint(DyadicSquare A,Deg100Polygon P) {
	Deg100IntegerPolygon AA=new Deg100IntegerPolygon(A);
	Deg100IntegerPolygon PP=new Deg100IntegerPolygon(P);
	return(checkDisjoint(AA,PP));
    }
	      


}