package Current.popups.Deg100;

import Current.popups.Unfold.*;
import Current.*;
import Current.basic.*;
import Current.plot.Dyadic.*;

import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;
import java.math.*;

public class Deg100SanityCheck {

    int[][] numerator=new int[3][500];           //numerator
    int[][][] denominator=new int[3][3][500];    //denominators of defining functions
    CombinatorialTriangle[] CT;                  //all the combinatorial info is encoded here
    int[] top;                                   //list of top vertices
    int[] bot;                                   //list of bottom vertices
    int spine;                                   //spine number for defining function
    VertexPair V;                                //indexes current defining function
    DyadicSquare A;
    String word;
    int palindrome;
    Deg100Polygon DP;

    
    public Deg100SanityCheck(Deg100Polygon DP) {
	this.word=DP.W;
        CT=CombinatorialTriangle.unfold(word);
	CT=CombinatorialTriangle.assignTails(CT);
        denominator[0]=Function.computeDenominator(0,CT);
        denominator[1]=Function.computeDenominator(1,CT);
        denominator[2]=Function.computeDenominator(2,CT);
        top=Spine.completeList(1,CT);
        bot=Spine.completeList(2,CT);
        int[] x={1,1};
        int[] y={1,1};
        A=new DyadicSquare(x,y,0);
        int qq=word.length()/2;
	palindrome=Converter.palindromeTest(CT);
        if(palindrome==1) qq=word.length()/4+1;
        A.top=initPlayers(qq);
        A.bot=initPlayers(qq);
	V=new VertexPair(0,0,0,0);
	this.DP=DP;
    }

    
    public void computeNumerator() {
        int[][]f=new int[0][0];
        spine=Converter.spineNumber(V,CT);
	numerator=Function.getSignedNumerator(denominator[spine],V,CT);
    }


    public int[] initPlayers(int q) {
        int[] play=new int[q+1];
        play[0]=q;
        for(int i=1;i<=q;++i) play[i]=i;
        return(play);
    }



    /**In this routine we choose the 0th vertex of the dyadic polygon associated 
       to the word and check two things for every defining function:

       1. we compute the defining function in two ways.  The first way is as in
          the verification algorithm. The second way is geometrically: we take
          the unfolding and directly compare the heights of the vertices.
          We check that the one value is a constant multiple of the other one,
          up to a small tolerance.  Thus, our combinatorial method is correct.

       2. The first 7 digits of 2^{106} times the numerical version of the defining function
          coinsides with the first 7 digits of the lower endpoint of the 
          BigInterval version of the function. This serves as
          evidence that we are computing things correctly.


      There are some restrictions on these checks:  We only run them if the absolute value of the
      geometrically computed quantity is at least .001. 
    */



    public int check1(int q) {
	System.out.println("sanify check1 "+q);
        double ratio1=0;
	double ratio2=0;
	double tol1=.001;
	double tol2=.000001;


        for(int i=1;i<=A.top[0];++i) {
            for(int j=1;j<=A.bot[0];++j) {
		int ii=A.top[i];
		int jj=A.bot[j];
                V.o1=1;V.e1=top[ii];
                V.o2=2;V.e2=bot[jj];
		computeNumerator();
		    DyadicRational X1=new DyadicRational(DP.k1[0],DP.n1[0]);
		    DyadicRational X2=new DyadicRational(DP.k2[0],DP.n2[0]);
		    Complex Z=DyadicRational.flipConvert(X1,X2);
		    double test0=Corridor.geometricEvaluate(word,V,Z,0);
		    if(Math.abs(test0)>tol1) {
                    double test1=Deg100Compute.evaluate(numerator,denominator[spine],X1,X2);    //NUMERICAL 
		    ratio1=test1/test0;
		    int fail=0;
                    if(test1/test0<0) fail=1; 
		    if((ratio2!=0)&&(Math.abs(ratio1-ratio2)>tol2)) fail=1; 
		    if(fail!=0) System.out.println("fail "+"a"+ii+" b"+jj);
		    if(fail!=0) System.out.println(test0+" "+test1+" "+ratio1+" "+ratio2);
		    if(fail!=0) return(0);
		    ratio2=ratio1;
		    }
	    }
	}


        for(int i=1;i<=A.top[0];++i) {
            for(int j=i+1;j<=A.top[0];++j) {
		int ii=A.top[i];
		int jj=A.top[j];
                V.o1=1;V.e1=top[ii];
                V.o2=1;V.e2=top[jj];
		computeNumerator();
	            DyadicRational X1=new DyadicRational(DP.k1[0],DP.n1[0]);
		    DyadicRational X2=new DyadicRational(DP.k2[0],DP.n2[0]);
                    Complex Z=DyadicRational.flipConvert(X1,X2);
                    double test0=Corridor.geometricEvaluate(word,V,Z,0);
		    if(Math.abs(test0)>tol1) {
                    double test1=Deg100Compute.evaluate(numerator,denominator[spine],X1,X2);    //NUMERICAL 
		    ratio1=test1/test0;
		    int fail=0;
		    if(test1/test0<0) fail=1;     
                    if((ratio2!=0)&&(Math.abs(ratio1-ratio2)>tol2)) fail=1; 
                    if(fail!=0) System.out.println("fail "+"a"+ii+" a"+jj);   
                    if(fail!=0) System.out.println(test0+" "+test1+" "+ratio1+" "+ratio2); 
		    if(fail!=0) return(0);  
                    ratio2=ratio1;
		    }

	    }
	}


        for(int i=1;i<=A.bot[0];++i) {
            for(int j=i+1;j<=A.bot[0];++j) {
		int ii=A.bot[i];
		int jj=A.bot[j];
                V.o1=2;V.e1=bot[ii];
                V.o2=2;V.e2=bot[jj];
		computeNumerator();	
                    DyadicRational X1=new DyadicRational(DP.k1[0],DP.n1[0]);
		    DyadicRational X2=new DyadicRational(DP.k2[0],DP.n2[0]);
                    Complex Z=DyadicRational.flipConvert(X1,X2);
                    double test0=Corridor.geometricEvaluate(word,V,Z,0);  
                    if(Math.abs(test0)>tol1) {
                    double test1=Deg100Compute.evaluate(numerator,denominator[spine],X1,X2);    //NUMERICAL
                    ratio1=test1/test0;
		    int fail=0;
		    if(test1/test0<0) fail=1;   
                    if((ratio2!=0)&&(Math.abs(ratio1-ratio2)>tol2)) fail=1; 
                    if(fail!=0) System.out.println("fail "+"b"+ii+" b"+jj);
                    if(fail!=0) System.out.println(test0+" "+test1+" "+ratio1+" "+ratio2);
		    if(fail!=0) return(0);
		    ratio2=ratio1;
		    }

	    }
	}
	return(1);

    }





    public int check2(int q) {

        System.out.println("sanity check2 "+q);
        double tol1=.001;


        for(int i=1;i<=A.top[0];i=i+5) {
            for(int j=1;j<=A.bot[0];j=j+5) {
		int ii=A.top[i];
		int jj=A.bot[j];
                V.o1=1;V.e1=top[ii];
                V.o2=2;V.e2=bot[jj];
		computeNumerator();
		    DyadicRational X1=new DyadicRational(DP.k1[0],DP.n1[0]);
		    DyadicRational X2=new DyadicRational(DP.k2[0],DP.n2[0]);
		    Complex Z=DyadicRational.flipConvert(X1,X2);
                    double test1=Deg100Compute.evaluate(numerator,denominator[spine],X1,X2);    //NUMERICAL 
                    if(Math.abs(test1)>tol1) {
		    String S2="";
		    String S3="";
                      BigInterval TEST2=Deg100Compute.evaluate2(numerator,denominator[spine],X1,X2);   //RIGOROUS
		      BigDecimal TEST1=new BigDecimal(test1);
		      TEST1=TEST1.multiply(new BigDecimal(new BigInteger("81129638414606681695789005144064")));
		      BigInteger TEST3=TEST1.toBigInteger();
		      S2=TEST2.L.toString();
		      S2=S2.substring(0,6);  
                      S3=TEST3.toString();
		      S3=S3.substring(0,6);

		    int fail=0;
		    if(S2.compareTo(S3)!=0) fail=1;
		    if(fail==1) System.out.println("interval");
		    if(fail!=0) return(0);
		    }
	    }
	}


        for(int i=1;i<=A.top[0];i=i+5) {
            for(int j=i+1;j<=A.top[0];j=j+5) {
		int ii=A.top[i];
		int jj=A.top[j];
                V.o1=1;V.e1=top[ii];
                V.o2=1;V.e2=top[jj];
		computeNumerator();
		    DyadicRational X1=new DyadicRational(DP.k1[0],DP.n1[0]);
		    DyadicRational X2=new DyadicRational(DP.k2[0],DP.n2[0]);
		    Complex Z=DyadicRational.flipConvert(X1,X2);
                    double test1=Deg100Compute.evaluate(numerator,denominator[spine],X1,X2);    //NUMERICAL 
                    if(Math.abs(test1)>tol1) {
		    String S2="";
		    String S3="";
                      BigInterval TEST2=Deg100Compute.evaluate2(numerator,denominator[spine],X1,X2);   //RIGOROUS
		      BigDecimal TEST1=new BigDecimal(test1);
		      TEST1=TEST1.multiply(new BigDecimal(new BigInteger("81129638414606681695789005144064")));
		      BigInteger TEST3=TEST1.toBigInteger();
		      S2=TEST2.L.toString();
		      S2=S2.substring(0,6);  
                      S3=TEST3.toString();
		      S3=S3.substring(0,6);

		    int fail=0;
		    if(S2.compareTo(S3)!=0) fail=1;
		    if(fail==1) System.out.println("interval");
		    if(fail!=0) return(0);
		    }
	    }
	}


        for(int i=1;i<=A.bot[0];i=i+5) {
            for(int j=i+1;j<=A.bot[0];j=j+5) {
		int ii=A.bot[i];
		int jj=A.bot[j];
                V.o1=2;V.e1=bot[ii];
                V.o2=2;V.e2=bot[jj];
		computeNumerator();
		    DyadicRational X1=new DyadicRational(DP.k1[0],DP.n1[0]);
		    DyadicRational X2=new DyadicRational(DP.k2[0],DP.n2[0]);
		    Complex Z=DyadicRational.flipConvert(X1,X2);
                    double test1=Deg100Compute.evaluate(numerator,denominator[spine],X1,X2);    //NUMERICAL 
                    if(Math.abs(test1)>tol1) {
		    String S2="";
		    String S3="";
                      BigInterval TEST2=Deg100Compute.evaluate2(numerator,denominator[spine],X1,X2);   //RIGOROUS
		      BigDecimal TEST1=new BigDecimal(test1);
		      TEST1=TEST1.multiply(new BigDecimal(new BigInteger("81129638414606681695789005144064")));
		      BigInteger TEST3=TEST1.toBigInteger();
		      S2=TEST2.L.toString();
		      S2=S2.substring(0,6);  
                      S3=TEST3.toString();
		      S3=S3.substring(0,6);

		    int fail=0;
		    if(S2.compareTo(S3)!=0) fail=1;
		    if(fail==1) System.out.println("interval");
		    if(fail!=0) return(0);
		    }
	    }
	}
	return(1);
    }







    public int check3(int q) {
	System.out.println("sanify check3 "+q);
        double ratio1=0;
	double ratio2=0;
	double tol1=.000001;
	double tol2=.001;


        for(int i=1;i<=A.top[0];i=i+5) {
            for(int j=1;j<=A.bot[0];j=j+5) {
		int ii=A.top[i];
		int jj=A.bot[j];
                V.o1=1;V.e1=top[ii];
                V.o2=2;V.e2=bot[jj];
		computeNumerator();
		    DyadicRational X1=new DyadicRational(DP.k1[0],DP.n1[0]);
		    DyadicRational X2=new DyadicRational(DP.k2[0],DP.n2[0]);
		    DyadicRational dX=new DyadicRational(1,30);
		    DyadicRational X3=DyadicRational.plus(X1,dX);
                    double test0=Deg100Compute.evaluate(numerator,denominator[spine],X1,X2);    
                    double test1=Deg100Compute.evaluate(numerator,denominator[spine],X3,X2);  
		    double test2=(test1-test0)*Math.pow(2,30);
		    test2=2.0*test2/Math.PI;
		    double test3=Deg100Compute.gradient(1,numerator,denominator[spine],X1,X2);
		    if(test2>tol1) {
		    double test4=Math.abs(test2-test3)/test2;
		    if(test4>tol2) {
			System.out.println("fail");
			return(0);
		    }
		    }
	    }
	}

        for(int i=1;i<=A.top[0];i=i+5) {
            for(int j=1;j<=A.bot[0];j=j+5) {
		int ii=A.top[i];
		int jj=A.bot[j];
                V.o1=1;V.e1=top[ii];
                V.o2=2;V.e2=bot[jj];
		computeNumerator();
		    DyadicRational X1=new DyadicRational(DP.k1[0],DP.n1[0]);
		    DyadicRational X2=new DyadicRational(DP.k2[0],DP.n2[0]);
		    DyadicRational dX=new DyadicRational(1,30);
		    DyadicRational X3=DyadicRational.plus(X2,dX);
                    double test0=Deg100Compute.evaluate(numerator,denominator[spine],X1,X2);    
                    double test1=Deg100Compute.evaluate(numerator,denominator[spine],X1,X3);  
		    double test2=(test1-test0)*Math.pow(2,30);
		    test2=2.0*test2/Math.PI;
		    double test3=Deg100Compute.gradient(2,numerator,denominator[spine],X1,X2);
		    if(test2>tol1) {
			double test4=Math.abs(test2-test3)/Math.abs(test2);
		    if(test4>tol2) {
			System.out.println("fail");
			return(0);
		    }
		    }
	    }
	}


	return(1);
    }







}



