/*
 * TileMemory.java
 *
 * Created on June 8, 2006, 2:33 PM
 */

package Current.popups.Memory;

import Current.*;
import Current.manage.*;
import Current.plot.Plotters.XPlotter;

import java.awt.Color;
import java.io.*;
import java.util.*;


/**
 * This class handles reading and writing tiles to files.
 * @author  pat
 */
public class TileMemory {
    
    /** STATIC METHODS handle saving */
    
    protected static final int
    WORDS_AND_COLORS=0,
    CONSTRUCTORS=1;
    
    /** takes a string containing the filename where you wish to store the tile data.
     ** "Files/Tiles/" will be prepended to the filename */
    public static void saveWordsAndColors(String filename, Tile[] tiles){
        try {
            FileOutputStream out=new FileOutputStream("Files/Tiles/"+filename);
            ObjectOutputStream objout=new ObjectOutputStream(out);
            
            // save the TileWrite type
            objout.writeInt(WORDS_AND_COLORS);
            objout.writeInt(tiles.length);
            for (int i=0; i<tiles.length; i++) {
                objout.writeObject(tiles[i].getStringWord());
                objout.writeObject(tiles[i].getColor());
            }
            objout.close();
            Process P=Runtime.getRuntime().exec("Files/java_update");
            P.waitFor();
        } catch (Throwable t) {
            System.err.println("Failed to save properties file.");
            t.printStackTrace(System.err);
        }
    }
    
    /** takes a string containing the filename where you wish to store the tile data.
     ** "Files/Tiles/" will be prepended to the filename */
    public static void saveConstructors(String filename, Tile[] tiles){
        try {
            FileOutputStream out=new FileOutputStream("Files/Tiles/"+filename);
            ObjectOutputStream objout=new ObjectOutputStream(out);
            
            // save the TileWrite type
            objout.writeInt(CONSTRUCTORS);
            objout.writeInt(tiles.length);
            for (int i=0; i<tiles.length; i++) {
                objout.writeObject(tiles[i].getStringWord());
                objout.writeObject(tiles[i].getColor());
                objout.writeObject(tiles[i].getPlotter());
            }
            objout.close();
            Process P=Runtime.getRuntime().exec("Files/java_update");
            P.waitFor();
        } catch (Throwable t) {
            System.err.println("Failed to save properties file.");
            t.printStackTrace(System.err);
        }
    }
    
    
    /** The rest handles loading */
    
    protected Manager M;
    protected int type;    
    protected ObjectInputStream obj_in;
    
    /** Creates a new instance of TileMemory */
    public TileMemory(Manager M, String filename) {
        this.M=M;
                
        type=-1;
        obj_in=null;
        
        try {
            obj_in=new ObjectInputStream(M.getInputStream("Files/Tiles/"+filename));
            type=obj_in.readInt();
        } catch (Throwable T) {
            System.out.println("Error opening object input stream");
            T.printStackTrace();
            return;
        }
    }
    
    public int getType() {
        return type;
    }
        
    
    public LinkedList load() {
        LinkedList tiles=new LinkedList();
        switch (type) {
            case WORDS_AND_COLORS:
                loadWordsAndColors(tiles);
                break;
            case CONSTRUCTORS:
                loadConstructors(tiles);
                break;
            default:
                System.err.println("Error: Load of type \""+type+"\" not supported.");
        }
        return tiles;
    }
    
    protected void loadWordsAndColors(LinkedList tiles){
        try {
            int length=obj_in.readInt();
            String w;
            Color C;
            for (int i=0; i<length; i++) {
                w=(String)(obj_in.readObject());
                C=(Color)(obj_in.readObject());
                tiles.addLast(new McbTile(w,C));
            }
            obj_in.close();
        } catch (Throwable T) {
            System.out.println("Error loading Words and Colors");
            T.printStackTrace();
            return;
        }
    }
    
    protected void loadConstructors(LinkedList tiles){
        try {
            int length=obj_in.readInt();
            String w;
            Color C;
            XPlotter P;
            for (int i=0; i<length; i++) {
                w=(String)(obj_in.readObject());
                C=(Color)(obj_in.readObject());
                try {
                    P=(XPlotter)(obj_in.readObject());
                } catch (Throwable T) {
                    System.err.println("Warning: failed to load a plotter");
                    P=null;
                }
                tiles.addLast(new McbTile(w,C,P));
            }
            obj_in.close();
        } catch (Throwable T) {
            System.out.println("Error loading constructors");
            T.printStackTrace();
            return;
        }
    }
    
}
