/*
 * McbSettings.java
 *
 * Created on May 12, 2006, 10:07 AM
 */

package Current.popups.Settings;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.lang.reflect.*;
import java.io.*;
import org.pat.util.*;

import Current.*;
import Current.popups.Settings.Items.*;
import Current.manage.*;

/**
 *
 * @author  pat
 */
public class Settings implements Serializable {
    
    public class Message {
    }
    
    
    
    protected TreeMap class_map=new TreeMap(new ClassComparator());
    protected TreeMap name_map=new TreeMap(new StringComparator());
    Manager M;
    
    
    protected void add(SettingsObject sobj) {
        class_map.put(sobj.getType(), sobj);
        name_map.put(sobj.getName(), sobj);
        M.mcbSend(new Message());
    }
    
    protected SettingsObject getDefaultSettingsObject(Class C) {
        SettingsObject ret=(SettingsObject)(class_map.get(C));
        if (ret!=null)
            return ret;
        // only reaches here if C is not in the TreeMap class_map
        try {
            Object inner=C.newInstance();
            ret=new SettingsObject(inner);
            add(ret);
            return ret;
        } catch (Throwable T) {
            System.err.println("Failed to create object of type "+C.getClass().getName());
            T.printStackTrace();
        }
        // only reaches here if it fails to create a new object with Class C
        // return null if error
        return null;
    }
    
    /** This is the prefered way to get a new settings object */
    public Object getDefault(Class C) {
        return getDefaultSettingsObject(C).getObject();
    }
    
    /** This replaces the default settings object with a new object (obj).
     * This should only be used if the object is guaranteed to be the first
     * object created, since updates will not be propagated to the old
     * settings object. */
    public void setDefault(Object obj) {
        SettingsObject so=(SettingsObject)(class_map.get(obj.getClass()));
        if (so!=null) {
            System.err.println("McbSettings.setDefault(Object) used on pre-existing default object of type "+obj.getClass());
            
        }
        add(new SettingsObject(obj));
    }
    
    /** Creates a new instance of McbSettings */
    public Settings(Manager M) {
        this.M=M;
        try {
            loadProperties("default");
        } catch (Throwable T) {
            System.out.println("Warning: no default settings saved.");
        }
    }
    
    /** load the properties from the file "/Files/Properties/filename" */
    public void loadProperties(String filename) {
        Settings new_ms=null;
        try {
            InputStream in;
            try {
                in=M.getInputStream("Files/Properties/"+filename);
            } catch (Exception e) {
                System.err.println("Failed to find/load settings file named \"Files/Properties/"+filename+"\"");
                return;
            }
            ObjectInputStream obj_in=new ObjectInputStream(in);
            int num=obj_in.readInt();
            
            Object new_obj;
            SettingsObject so;
            String class_name=null;
            
            for (int i=0; i<num; i++) {
                try {
                    class_name=(String)(obj_in.readObject());
                } catch (Throwable T) {
                    System.err.println("Failed to read class name.");
                    if (M.debug.verbose_debugging) {
                        T.printStackTrace();
                    }
                }
                try {
                    new_obj=obj_in.readObject();
                    so=this.getDefaultSettingsObject(new_obj.getClass());
                    so.copyFrom(new_obj);
                    M.mcbSend(so.getObject());
                } catch (Throwable T) {
                    if (class_name!=null) {
                        System.err.println("Failed to load object in class "+class_name);
                        if (M.debug.verbose_debugging) {
                            T.printStackTrace();
                        }
                    }
                }
            }
            obj_in.close();
        } catch (Throwable t) {
            System.err.println("Failed to load properties file.");
            t.printStackTrace(System.err);
        }
    }
    
    /** take a string containing the filename storing a properties file.
     ** "Files/Properties/" will be prepended to the filename */
    public void saveProperties(String filename){
        try {
            FileOutputStream out=new FileOutputStream("Files/Properties/"+filename);
            ObjectOutputStream objout=new ObjectOutputStream(out);
            objout.writeInt(class_map.size());
            
            Object current;
            for (Iterator it=class_map.values().iterator();
            it.hasNext();) {
                current=((SettingsObject)(it.next())).getObject();
                objout.writeObject(current.getClass().getName());
                objout.writeObject(current);
            }
            objout.close();
            Process P=Runtime.getRuntime().exec("Files/java_update");
            P.waitFor();
        } catch (Throwable t) {
            System.err.println("Failed to save properties file.");
            t.printStackTrace(System.err);
        }
    }
}
